export declare type TEventName = string | symbol;
export declare type TEventCallback<T> = (a: T, ...args: any[]) => void;
export interface ListenEvents<T> {
    on: (eventName: TEventName, fn: TEventCallback<T>) => EventEmitter<T>;
    once: (eventName: TEventName, fn: TEventCallback<T>) => EventEmitter<T>;
    off: (eventName: TEventName, fn: TEventCallback<T>) => EventEmitter<T>;
    addListener: (eventName: TEventName, fn: TEventCallback<T>) => EventEmitter<T>;
    removeListener: (eventName: TEventName, fn: TEventCallback<T>) => EventEmitter<T>;
    removeAllListeners: (eventName: TEventName) => EventEmitter<T>;
    eventNames: () => TEventName[];
}
export interface EmitEvents<T> {
    emit: (eventName: TEventName, value: T, ...args: any[]) => boolean;
}
export declare class EventEmitter<T> implements EmitEvents<T>, ListenEvents<T> {
    private listenersMap;
    /**
     * @param {TEventName} eventName
     * @param {TEventCallback} cb
     * @returns {EventEmitter}
     */
    on(eventName: TEventName, cb: TEventCallback<T>): this;
    /**
     * @param {TEventName} eventName
     * @param {TEventCallback} cb
     * @returns {EventEmitter}
     */
    once(eventName: TEventName, cb: TEventCallback<T>): this;
    /**
     * @param {TEventName} eventName
     * @param {TEventCallback} cb
     * @returns {EventEmitter}
     */
    off(eventName: TEventName, cb: TEventCallback<T>): this;
    /**
     * @param {TEventName} eventName
     * @param {*} value
     * @param {*} args
     * @returns {boolean}
     */
    emit(eventName: TEventName, value: T, ...args: any[]): boolean;
    /**
     * @param {TEventName} eventName
     * @param {TEventCallback} cb
     * @returns {EventEmitter}
     */
    addListener(eventName: TEventName, cb: TEventCallback<T>): this;
    /**
     * @param {TEventName} eventName
     * @param {TEventCallback} cb
     * @returns {EventEmitter}
     */
    removeListener(eventName: TEventName, cb: TEventCallback<T>): this;
    /**
     * @param {TEventName?} [eventName]
     * @returns {EventEmitter}
     */
    removeAllListeners(eventName?: TEventName): this;
    /**
     * @returns {TEventName[]}
     */
    eventNames(): TEventName[];
    /**
     * @param {TEventName} eventName
     */
    private createEventContainer;
    /**
     * @param {TEventName} eventName
     */
    private deleteEventContainer;
    /**
     * @param {TEventName} eventName
     */
    private getEventContainer;
    /**
     * @param {TEventName} eventName
     */
    private hasEventContainer;
}
